/*
Copyright (c) 1994, The Regents of the University of Colorado

Permission to use, copy, modify, distribute, and sell this software and
its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the University of
Colorado not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.  The University of Colorado makes no representations about
the suitability of this software for any purpose.  It is provided "as
is" without express or implied warranty.

The University of Colorado disclaims all warranties with regard to this
software, including all implied warranties of merchantability and
fitness, in no event shall the University of Colorado be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortious action, arising out of or in
connection with the use or performance of this software.
*/

#include <stdio.h>

/* ++pac
#include <vera2pov.h>
*/
#include "vera2pov.h"

int ReadVeraFile (verafile,ntriangles,triangles,normals,verbose,really_verbose)
  char 			*verafile;
  unsigned long int	*ntriangles;
  float			**triangles;
  float			**normals;
  short			verbose;
  short			really_verbose;
{
FILE			*fp;
int			v,c,status,cmtlvl;
unsigned long int	i=0;
char			ch,tmpstr[256];
float			f,*t,*n;

/* open file */
if ((fp = fopen(verafile,"r")) == NULL) {
  return(FILE_UNOPENED);
}

if (verbose) fprintf(stdout,"Reading vera file...");
/* count number of uncommented strings "PA" to get number of triangles */
/* this implementation allows for nested comments (* ... (* ... *) ... *) */
i = (unsigned long) 0;
cmtlvl = 0;
while (!feof(fp)) {
  if ((ch = fgetc(fp)) == EOF) break;
  if ((ch == '(') && ((ch=fgetc(fp)) == '*')) cmtlvl++;
  if ((ch == '*') && ((ch=fgetc(fp)) == ')')) cmtlvl--;
  if ((cmtlvl == 0) && (ch == 'P') && ((ch=fgetc(fp)) == 'A')) i++;
}
if ((*ntriangles = i) == 0) {
  fclose(fp);
  return(NO_TRIANGLES);
}
if (verbose) fprintf(stdout,"done.\nNumber of triangles:  %d.\n",*ntriangles);

/* allocate space for arrays triangles & normals */
if ((*triangles = (float *) calloc(3*3*(*ntriangles),sizeof(float))) == NULL) {
  fclose(fp);
  return(NOT_ENOUGH_MEMORY);
}
if ((*normals = (float *) calloc(3*3*(*ntriangles),sizeof(float))) == NULL) {
  fclose(fp);
  free(triangles);
  return(NOT_ENOUGH_MEMORY);
}

/* reset file pointer */
rewind(fp);

/* fill triangles & normals */
cmtlvl = 0;
t = *triangles;
n = *normals;
i = 0;
while (!feof(fp) && (i < *ntriangles)) {
  if ((ch = fgetc(fp)) == EOF) {
    fclose(fp);
    free(triangles);
    free(normals);
    return(PREMATURE_EOF);
  }
  if ((ch == '(') && ((ch=fgetc(fp)) == '*')) cmtlvl++;
  if ((ch == '*') && ((ch=fgetc(fp)) == ')')) cmtlvl--;
  if ((cmtlvl == 0) && (ch == 'P') && ((ch=fgetc(fp)) == 'A')) {
    i++;
    if (verbose)
      fprintf(really_verbose ? stdout : stderr,
        "Reading triangle %d%s",i,really_verbose ? ":\n" : ".\r");
    /* for each vertex & for each coordinate, read into triangle space */
    for (v = 0; v < 3; v++) {
      if (really_verbose) fprintf(stdout,"Vtx %d:  ",v+1);
      for (c = 0; c < 3; c++) {
        if (fscanf(fp,"%f",&f) != 1) {
          fclose(fp);
          free(triangles);
          free(normals);
          return(FORMAT_ERROR);
        }
        if (really_verbose) fprintf(stdout,"%f ",f);
        *t++ = f;
      }
      if (really_verbose) fprintf(stdout,"\n");
    }
    /* now do same to fill normal space */
    for (v = 0; v < 3; v++) {
      if (really_verbose) fprintf(stdout,"Norm %d:  ",v+1);
      for (c = 0; c < 3; c++) {
        if (fscanf(fp,"%f",&f) != 1) {
          fclose(fp);
          free(triangles);
          free(normals);
          return(FORMAT_ERROR);
        }
        if (really_verbose) fprintf(stdout,"%f ",f);
        *n++ = f;
      }
      if (really_verbose) fprintf(stdout,"\n");
    }
    if (really_verbose) fprintf(stdout,"\n");
  }
}
if (verbose) fprintf(stdout,"\n");
fclose(fp);
return(SUCCESS);
}
