/*
Copyright (c) 1994, The Regents of the University of Colorado

Permission to use, copy, modify, distribute, and sell this software and
its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the University of
Colorado not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.  The University of Colorado makes no representations about
the suitability of this software for any purpose.  It is provided "as
is" without express or implied warranty.

The University of Colorado disclaims all warranties with regard to this
software, including all implied warranties of merchantability and
fitness, in no event shall the University of Colorado be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortious action, arising out of or in
connection with the use or performance of this software.
*/

#include <stdio.h>
/* ++pac
#include <vera2pov.h>
*/
#include "vera2pov.h"

#define USAGE	"usage:\n %s \\\n  [-h[elp]] \\\n  [-s[mooth_triangles]] \\\n  [-t[riangles]] \\\n  [-c[enter]] \\\n  [-q[uiet] | -v[erbose] | -r[eally_verbose]] \\\n  [-n[ame] object_name] \\\n  [-m[atrix] matrix_file] \\\n  verafile \\\n  povfile\n"

void Usage(callname)
  char *callname;
{
fprintf(stderr, USAGE, (callname));
}

int ReadMtxFile(mtxfile,mtx)
  char	*mtxfile;
  float	mtx[4][4];
{
FILE	*fp;
short	i,j;
float	f;

if ((fp = fopen(mtxfile,"r")) == NULL)
  return(FILE_UNOPENED);

for (i = 0; i < 4; i++) {
  for (j = 0; j < 4; j++) {
    if (fscanf(fp,"%f",&f) != 1) {
      fclose(fp);
      return(MTX_FORMAT_ERR);
    }
    mtx[i][j] = f;
  }
}
return(SUCCESS);
}


main(argc, argv)
  int	argc;
  char	**argv;
{
short	smooth=TRUE,verbose=TRUE,really=FALSE,quiet=FALSE,center=FALSE;
char	*callname,*verafile=NULL,*povfile=NULL,*name=NULL,*mtxfile=NULL;
int	i;
long	ntriangles;
float	*triangles=NULL,*normals=NULL,*mtx=NULL;

if ((argc < 2) || (argc > 9)) {
  Usage(argv[0]);
  exit(UNKNOWN_FILENAME);
}

/* parse command line */
callname = argv[0];
while (*(++argv) != NULL) {
  if (**argv == '-') {
         if (!strncmp(*argv,"-s",2)) smooth = TRUE;
    else if (!strncmp(*argv,"-t",2)) smooth = FALSE;
    else if (!strncmp(*argv,"-c",2)) center = TRUE;
    else if (!strncmp(*argv,"-v",2)) {
      verbose = TRUE;
      quiet = FALSE;
    }
    else if (!strncmp(*argv,"-r",2)) {
      verbose = TRUE;
      really = TRUE;
      quiet = FALSE;
    }
    else if (!strncmp(*argv,"-q",2)) {
      quiet = TRUE;
      verbose = FALSE;
      really = FALSE;
    }
    else if (!strncmp(*argv,"-n",2)) {
      if (++argv != NULL)
        name = *argv;
      else {
        Usage(callname);
        exit(NO_OBJ_NAME);
      }
    }
    else if (!strncmp(*argv,"-m",2)) {
      if (++argv != NULL)
        mtxfile = *argv;
      else {
        Usage(callname);
        exit(NO_MTX_FILENAME);
      }
    }
    else if (!strncmp(*argv,"-h",2)) {
      if (Header(stderr,"") != SUCCESS) exit(FAILURE);
      if (fprintf(stderr,"\n") < 0) exit(FAILURE);
      Usage(callname);
      exit(SUCCESS);
    }
    else {
      Usage(callname);
      exit(UNKNOWN_OPTION);
    }
  }
  else {
    if (verafile == NULL) verafile = *argv;
    else if (povfile == NULL) povfile = *argv;
    else {
      Usage(callname);
      exit(UNKNOWN_FILENAME);
    }
  }
}

if (!quiet)
  if (Header(stdout,"") != SUCCESS) exit(FAILURE);

if (mtxfile != NULL) {
  if ((mtx = (float *) calloc(4*4,sizeof(float))) == NULL)
    exit(NOT_ENOUGH_MEMORY);
  if ((i=ReadMtxFile(mtxfile,mtx)) != SUCCESS) {
    fprintf(stderr,"Error reading matrix file %s.\n",mtxfile);
    exit(i);
  }
}

if (verbose) {
  fprintf(stdout,"%ss will be written.\n", smooth ? SMOOTH_TRIANGLE : TRIANGLE);
  fprintf(stdout,"vera file:\t%s\n",verafile);
  fprintf(stdout,"PoV file:\t%s\n",povfile);
  if (mtxfile != NULL)
    fprintf(stdout,"matrix file:\t%s\n",mtxfile);
}
if (name == NULL) name = DEFAULT_OBJ_NAME;

/* read verafile */
if ((i=ReadVeraFile(verafile,&ntriangles,&triangles,&normals,verbose,really))
     != SUCCESS) {
  fprintf(stderr,"Error reading vera file %s\n.",verafile);
  exit(i);
}

/* write povfile */
if (
  (i=WritePOV2File(
      povfile,name,ntriangles,triangles,(smooth ? normals : NULL),mtx,center,
      verbose,really))
    != SUCCESS) {
    fprintf(stderr,"Error writing PoV file %s\n.",povfile);
    exit(i);
  }

free(triangles);
free(normals);
if (mtx != NULL) free(mtx);
exit(SUCCESS);
}
